/*
 * adc.c
 *
 *  Created on: 08.03.2012
 *      Author: sn
 */

#include "adc.h"
#include "stdio.h"
#include "queue.h"
#include "oled.h"

#include "math.h"
#include "arm_math.h"

xQueueHandle xQueue1;

#define AD_SAMPLE_COUNT  2048
#define AD_SAMPLE_FREQ	2048 /* Hz */

float32_t *pWriteBuffer;
float32_t *analogValues;
float32_t *testOutput;
int adCount = 0;
char webmsg[20];

void ADC_IRQHandler(void)
{
	//GPIO_SetValue(0, (1<<6));

	portBASE_TYPE xHigherPriorityTaskWoken;

	// We have not woken a task at the start of the ISR
    xHigherPriorityTaskWoken = pdFALSE;

    //uint32_t stat = LPC_ADC->ADSTAT;
	uint32_t stat = ADC_GlobalGetStatus(LPC_ADC, 1);
	stat=stat;

	pWriteBuffer[adCount] = (float32_t)ADC_ChannelGetData(LPC_ADC, ADC_CHANNEL_5);

	adCount++;

	if (adCount >= AD_SAMPLE_COUNT)
	{
		TIM_Cmd(LPC_TIM1, DISABLE);
		xQueueSendFromISR( xQueue1, &pWriteBuffer, &xHigherPriorityTaskWoken );

		adCount = 0;
	}

	LPC_TIM1->EMR &= ~(1<<1);	// Reset ExtMatch Bit

	//GPIO_ClearValue(0, (1<<6));

    if( xHigherPriorityTaskWoken )
    {
    	// wake up higher prio task
        taskYIELD();
    }
}

static void init_timer(void)
{
	//timer init
	TIM_TIMERCFG_Type TIM_ConfigStruct;
	TIM_MATCHCFG_Type TIM_MatchConfigStruct ;

	// Initialize timer 0, prescale count time of 10uS
	TIM_ConfigStruct.PrescaleOption = TIM_PRESCALE_USVAL;
	TIM_ConfigStruct.PrescaleValue	= 1;

	// use channel 1, MR1
	TIM_MatchConfigStruct.MatchChannel = 1;
	// Enable interrupt when MR1 matches the value in TC register
	TIM_MatchConfigStruct.IntOnMatch   = FALSE;
	//Enable reset on MR0: TIMER will reset if MR0 matches it
	TIM_MatchConfigStruct.ResetOnMatch = TRUE;
	//Stop on MR0 if MR0 matches it
	TIM_MatchConfigStruct.StopOnMatch  = FALSE;
	//Toggle MR0.0 pin if MR0 matches it
	TIM_MatchConfigStruct.ExtMatchOutputType = TIM_EXTMATCH_HIGH;
	// Set Match value, count value of 10 (50 * 1uS = 50us  --> 20 kHz)
	TIM_MatchConfigStruct.MatchValue   = 1e6 / AD_SAMPLE_FREQ - 1;

	// Set configuration for Tim_config and Tim_MatchConfig
	TIM_Init(LPC_TIM1, TIM_TIMER_MODE, &TIM_ConfigStruct);
	TIM_ConfigMatch(LPC_TIM1, &TIM_MatchConfigStruct);

	// To start timer 1
	TIM_Cmd(LPC_TIM1, ENABLE);

	//LPC_PINCON->PINSEL3 |=(0x03L<<18);
	//PINSEL_SetPinFunc(1, 18, 3);
}

static void init_adc(void)
{
	PINSEL_CFG_Type PinCfg;

	// Testouput
	PinCfg.Funcnum = PINSEL_FUNC_0;
	PinCfg.OpenDrain = PINSEL_PINMODE_NORMAL;
	PinCfg.Pinmode = PINSEL_PINMODE_PULLUP;
	PinCfg.Portnum = PINSEL_PORT_0;
	PinCfg.Pinnum = PINSEL_PIN_6;
	PINSEL_ConfigPin(&PinCfg);
	GPIO_SetDir(0, (1<<6), 1);


	/*
	 * Init ADC pin connect
	 * AD0.5 on P1.31
	 */
	PinCfg.Funcnum = 3;
	PinCfg.OpenDrain = 0;
	PinCfg.Pinmode = 0;
	PinCfg.Pinnum = 31;
	PinCfg.Portnum = 1;
	PINSEL_ConfigPin(&PinCfg);


	/* Configuration for ADC :
	 * 	Frequency at 100kHz
	 *  ADC channel 5, Interrupt enabled, start on Timer1 Match1
	 */

	ADC_Init(LPC_ADC, 200000);
	ADC_IntConfig(LPC_ADC, ADC_ADINTEN5, SET);
	ADC_ChannelCmd(LPC_ADC, ADC_CHANNEL_5, ENABLE);
	ADC_EdgeStartConfig(LPC_ADC, ADC_START_ON_RISING);
	ADC_StartCmd(LPC_ADC, ADC_START_ON_MAT11);

	//ADC_BurstCmd(LPC_ADC, ENABLE);
	NVIC_SetPriority( ADC_IRQn, configADC_INTERRUPT_PRIORITY );
	NVIC_EnableIRQ(ADC_IRQn);

}

void drawArrayData(float32_t *pData)
{
	char cBuffer[32];

	uint32_t fftSize = AD_SAMPLE_COUNT/2;
	uint32_t ifftFlag = 0;
	uint32_t doBitReverse = 1;

	arm_status status;
	arm_cfft_radix4_instance_f32 S;
	float32_t maxValue;
	uint32_t testIndex = 0;

	status = ARM_MATH_SUCCESS;

	/* Initialize the CFFT/CIFFT module */
	status = arm_cfft_radix4_init_f32(&S, fftSize, ifftFlag, doBitReverse);

	/* Process the data through the CFFT/CIFFT module */
	arm_cfft_radix4_f32(&S, pData);


	/* Process the data through the Complex Magnitude Module for
	calculating the magnitude at each bin */
	arm_cmplx_mag_f32(pData, testOutput, fftSize);

	/* Calculates maxValue and returns corresponding BIN value */
	testOutput[0] = 0.0;
	arm_max_f32(testOutput, fftSize/2, &maxValue, &testIndex);
	float32_t freq = testIndex * (float32_t)AD_SAMPLE_FREQ / AD_SAMPLE_COUNT;

    //oled_fillRect(0, 16, 127, 63, OLED_COLOR_WHITE);
    oled_clearScreen(OLED_COLOR_BLACK);
    sprintf(cBuffer, "max: %7.1f %s", freq, webmsg);
    //sprintf(cBuffer, "max:%d", testIndex);
    oled_putString(0, 1*8,  (unsigned char*)cBuffer, OLED_COLOR_WHITE, OLED_COLOR_BLACK);

    maxValue = testOutput[testIndex];
    //maxValue = 4096.0;
    int x;
    for (x=0; x<128; x++)
    {
    	float32_t f,f1,f2;
    	f1 = MAX(testOutput[4*x+0], testOutput[4*x+1]);
    	f2 = MAX(testOutput[4*x+2], testOutput[4*x+3]);
    	f = MAX(f1, f2);
    	f = MIN(maxValue, f);
    	oled_line(x, 63, x, 63-(int)(f*48.0/maxValue), OLED_COLOR_WHITE);

    	//f = pData[x];
    	//oled_putPixel(x, 63-(int)(f*48.0/maxValue), OLED_COLOR_WHITE);
    }
}


portTASK_FUNCTION( vADCTask, pvParameters )
{
	portTickType xLastExecTime;

	/* The parameters are not used. */
	( void ) pvParameters;

	analogValues = pvPortMalloc(AD_SAMPLE_COUNT * sizeof(float32_t));
	testOutput = pvPortMalloc(AD_SAMPLE_COUNT/2 * sizeof(float32_t));
	pWriteBuffer = analogValues;

	// Create a queue capable of containing 10 unsigned long values.
	xQueue1 = xQueueCreate( 10, sizeof( unsigned portLONG ) );
	if( xQueue1 == 0 )
	{
		// Queue was not created and must not be used.
	}

	// initialize hardware
	init_timer();
    init_adc();

	/* We need to initialise xLastFlashTime prior to the first call to
	vTaskDelayUntil(). */
	xLastExecTime = xTaskGetTickCount();

    uint32_t memfree = xPortGetFreeHeapSize();
    printf("ADC Task heap free:%d\n", memfree);

	for(;;)
	{
		char cBuffer[32];
    	float_t *pData;
    	int i;

    	// wait for message, forever
		if( xQueueReceive( xQueue1, &pData, portMAX_DELAY))
		{
	    	GPIO_SetValue(0, (1<<6));
	    	//uint32_t timerStart = LPC_TIM0->TC;

		    drawArrayData(pData);

	    	//uint32_t timerDiff = LPC_TIM0->TC - timerStart;

			TIM_Cmd(LPC_TIM1, ENABLE);
		    GPIO_ClearValue(0, (1<<6));
		}


		//vTaskDelayUntil( &xLastExecTime, 200 / portTICK_RATE_MS );
	}
} /*lint !e715 !e818 !e830 Function definition must be standard for task creation. */

